@@ -13,14 +13,14 @@
 
 #include <qpa/qplatformwindow_p.h>
 
+#include <QWaylandClientExtension>
 #include <QWaylandClientExtensionTemplate>
-#include <qwaylandclientextension.h>
 
 #include "qwayland-blur.h"
 #include "qwayland-contrast.h"
 #include "qwayland-slide.h"
 
-#include "surfacehelper.h"
+#include "helpers.h"
 
 #include <wayland-client-protocol.h>
 
@@ -61,7 +61,9 @@ public:
 
     ~Blur() override
     {
-        release();
+        if (isQpaAlive()) {
+            release();
+        }
     }
 };
 
@@ -85,7 +87,9 @@ public:
 
     ~Contrast() override
     {
-        release();
+        if (isQpaAlive()) {
+            release();
+        }
     }
 };
 
@@ -109,7 +113,9 @@ public:
 
     ~Slide() override
     {
-        release();
+        if (isQpaAlive()) {
+            release();
+        }
     }
 };
 
@@ -263,13 +269,26 @@ bool WindowEffects::isEffectAvailable(KWindowEffects::Effect effect)
 void WindowEffects::slideWindow(QWindow *window, KWindowEffects::SlideFromLocation location, int offset)
 {
     if (location != KWindowEffects::SlideFromLocation::NoEdge) {
-        m_slideMap[window] = SlideData{
+        trackWindow(window);
+
+        const SlideData data{
             .location = location,
             .offset = offset,
         };
-        trackWindow(window);
+
+        if (auto it = m_slideMap.find(window); it != m_slideMap.end()) {
+            if (*it == data) {
+                return;
+            } else {
+                *it = data;
+            }
+        } else {
+            m_slideMap.insert(window, data);
+        }
     } else {
-        m_slideMap.remove(window);
+        if (!m_slideMap.remove(window)) {
+            return;
+        }
         releaseWindow(window);
     }
 
@@ -316,10 +335,21 @@ void WindowEffects::enableBlurBehind(QWindow *window, bool enable, const QRegion
 {
     if (enable) {
         trackWindow(window);
-        m_blurRegions[window] = region;
+
+        if (auto it = m_blurRegions.find(window); it != m_blurRegions.end()) {
+            if (*it == region) {
+                return;
+            } else {
+                *it = region;
+            }
+        } else {
+            m_blurRegions.insert(window, region);
+        }
     } else {
+        if (!m_blurRegions.remove(window)) {
+            return;
+        }
         resetBlur(window);
-        m_blurRegions.remove(window);
         releaseWindow(window);
     }
 
@@ -356,13 +386,28 @@ void WindowEffects::enableBackgroundContrast(QWindow *window, bool enable, qreal
 {
     if (enable) {
         trackWindow(window);
-        m_backgroundConstrastRegions[window].contrast = contrast;
-        m_backgroundConstrastRegions[window].intensity = intensity;
-        m_backgroundConstrastRegions[window].saturation = saturation;
-        m_backgroundConstrastRegions[window].region = region;
+
+        const BackgroundContrastData data{
+            .contrast = contrast,
+            .intensity = intensity,
+            .saturation = saturation,
+            .region = region,
+        };
+
+        if (auto it = m_backgroundConstrastRegions.find(window); it != m_backgroundConstrastRegions.end()) {
+            if (*it == data) {
+                return;
+            } else {
+                *it = data;
+            }
+        } else {
+            m_backgroundConstrastRegions.insert(window, data);
+        }
     } else {
+        if (!m_backgroundConstrastRegions.remove(window)) {
+            return;
+        }
         resetContrast(window);
-        m_backgroundConstrastRegions.remove(window);
         releaseWindow(window);
     }
 
