@@ -6,9 +6,9 @@
 */
 
 #include "windowshadow.h"
+#include "helpers.h"
 #include "logging.h"
 #include "shm.h"
-#include "surfacehelper.h"
 
 #include <qwayland-shadow.h>
 
@@ -16,7 +16,7 @@
 #include <QExposeEvent>
 #include <QWaylandClientExtension>
 
-#include <private/qwaylandwindow_p.h>
+#include <qpa/qplatformwindow_p.h>
 
 class ShadowManager : public QWaylandClientExtensionTemplate<ShadowManager>, public QtWayland::org_kde_kwin_shadow_manager
 {
@@ -38,7 +38,7 @@ class ShadowManager : public QWaylandClientExtensionTemplate<ShadowManager>, pub
 public:
     ~ShadowManager()
     {
-        if (isActive()) {
+        if (isQpaAlive() && isActive()) {
             destroy();
         }
     }
@@ -53,16 +53,19 @@ class Shadow : public QtWayland::org_kde_kwin_shadow
 {
 public:
     using QtWayland::org_kde_kwin_shadow::org_kde_kwin_shadow;
+
     ~Shadow()
     {
-        destroy();
+        if (isQpaAlive()) {
+            destroy();
+        }
     }
 };
 
 WindowShadowTile::WindowShadowTile()
 {
     connect(Shm::instance(), &Shm::activeChanged, this, [this] {
-        if (Shm::instance()->isActive()) {
+        if (!Shm::instance()->isActive()) {
             buffer.reset();
         }
     });
@@ -139,9 +142,9 @@ bool WindowShadow::internalCreate()
     }
 
     shadow = std::make_unique<Shadow>(ShadowManager::instance()->create(surface));
-    auto waylandWindow = dynamic_cast<QtWaylandClient::QWaylandWindow *>(window->handle());
+    auto waylandWindow = window->nativeInterface<QNativeInterface::Private::QWaylandWindow>();
     if (waylandWindow) {
-        connect(waylandWindow, &QtWaylandClient::QWaylandWindow::wlSurfaceDestroyed, this, &WindowShadow::internalDestroy, Qt::UniqueConnection);
+        connect(waylandWindow, &QNativeInterface::Private::QWaylandWindow::surfaceDestroyed, this, &WindowShadow::internalDestroy, Qt::UniqueConnection);
     }
 
     auto attach = [](const std::unique_ptr<Shadow> &shadow, auto attach_func, const KWindowShadowTile::Ptr &tile) {
@@ -188,6 +191,10 @@ void WindowShadow::internalDestroy()
         return;
     }
 
+    if (!isQpaAlive()) {
+        return;
+    }
+
     // Only call surfaceForWindow and unset the surface if the native window is alive.
     // Otherwise window->create() might be called when the window is being destroyed, leading to a crash.
     if (window && window->nativeInterface<QNativeInterface::Private::QWaylandWindow>() && ShadowManager::instance()->isActive()) {
